@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_93:mkdefs.sh,v 1.49 1997/04/12 01:25:17 woods Exp"
+#ident	"@(#)smail/conf/lib:RELEASE-3_2_0_94:mkdefs.sh,v 1.50 1997/04/21 18:49:02 woods Exp"
 
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
@@ -42,6 +42,10 @@ VARS="\
  DOT_Z\
  DRIVER_CONFIGURATION\
  FLOCK_MAILBOX\
+ FORCE_PATHS_FILE\
+ FORCE_PATHS_TYPE\
+ FORCE_SMTP_FILE\
+ FORCE_SMTP_TYPE\
  GATEWAY_NAMES\
  HOSTNAMES\
  LIB_DIR\
@@ -54,6 +58,8 @@ VARS="\
  LOCK_BY_NAME\
  LOG_DIR\
  MAILBOX_DIR\
+"
+VARS="$VARS\
  MAN1\
  MAN1_EXT\
  MAN5\
@@ -61,13 +67,9 @@ VARS="\
  MAN8\
  MAN8_EXT\
  MORE_HOSTNAMES\
-"
-VARS="$VARS\
  NEWALIASES\
  NEWS_SPOOL_DIR\
  SMAIL_NOBODY\
- PATH_ALIASES_FILE\
- PATH_ALIASES_TYPE\
  POINTER_TYPE\
  POSTMASTER\
  RANLIB\
@@ -85,6 +87,8 @@ VARS="$VARS\
  STRIP\
  TMP_DIR\
  UNCOMPRESS\
+"
+VARS="$VARS\
  UNSHAR_MAP_DIR\
  USE_ASCII\
  USE_SYMLINKS\
@@ -129,6 +133,9 @@ HAVE_REQUIRE="\
 # list vars that are set below, and *not* in EDITME/OS/ARCH, etc.
 # but are set by this script from derrived values....
 NOEDVARS="\
+ PATH_ALIASES_FILE\
+ PATH_FORCE_PATHS_FILE\
+ PATH_FORCE_SMTP_FILE\
  UTIL_PATH\
 "
 
