@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:bindlib.c,v 1.30 1997/04/12 20:52:26 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:bindlib.c,v 1.31 1997/04/21 18:01:10 woods Exp"
  */
 
 /*
@@ -442,18 +442,30 @@ bind_compute_domain ()
 /*
  * bind_check_if_local_mxs - check if localhost is MXing for target
  *
- * returns TRUE if any of the local hostnames is found in the MX records
- * for the target name.
- * returns FALSE with *defer set to TRUE if DNS fails.
- * returns FALSE with *defer set to FALSE if nothing matched.
+ * Return one of the following values:
+ *
+ * These return codes apply only to the specific address:
+ *      DB_SUCCEED+1	Yes, we MX for the target host.
+ *	DB_SUCCEED	No, we do not MX for the target host.
+ *      DB_NOMATCH	There is no such domain.
+ *      DB_FAIL		Negative response from server indicating some
+ *			problem with the request.
+ *      DB_AGAIN	Negative respone from server indicating a
+ *			temporary problem.  Try again later.
+ *
+ * These return codes apply to this router in general:
+ *      FILE_NOMATCH	Could not connect to the server.
+ *      FILE_AGAIN	Lost contact with server, or other failure
+ *			talking to server.  Try again later.
+ *      FILE_FAIL	A major error has been caught in router,
+ *			notify postmaster.
  */
 
 #define LOCALHOST_MX_CHECK	"localhost MX check"
 
 int
-bind_check_if_local_mxs(cur, defer_p)
+bind_check_if_local_mxs(cur)
     struct addr *cur;
-    int *defer_p;
 {
     long save_res_options;		/* like _res.options in <resolv.h> */
     char *strip_target;			/* target stripped of extra dots */
@@ -462,7 +474,6 @@ bind_check_if_local_mxs(cur, defer_p)
     int found_server = FALSE;		/* TRUE if server process found */
     int ret;
 
-    assert(defer_p);
 #ifdef RES_INIT
     /*  This initialises the resolver.  If the resolver is so old that RES_INIT
      *  isn't defined then the routines will initialise themselves - that is
@@ -472,8 +483,7 @@ bind_check_if_local_mxs(cur, defer_p)
 	if (res_init() == EOF) {
 	    DEBUG(DBG_DRIVER_LO, "ERROR: res_init() failed - defering addresses\n");
 	    cur->error = res_init_error(LOCALHOST_MX_CHECK);
-	    *defer_p = TRUE;
-	    return FALSE;
+	    return FILE_AGAIN;
 	}
     }
 #endif
@@ -498,31 +508,34 @@ bind_check_if_local_mxs(cur, defer_p)
     strip_target = strip_dots(cur->target); /* Omit needless dots! */
 
     mxdat = mx_work_area_constructor((struct bindlib_private *) NULL,
-				     cur->error, LOCALHOST_MX_CHECK, 0L,
+				     &cur->error, LOCALHOST_MX_CHECK, 0L,
 				     &found_server, &no_server);
 
-    if (find_mx_records(strip_target, mxdat, FALSE) != DB_SUCCEED) {
+    if ((ret = find_mx_records(strip_target, mxdat, FALSE)) != DB_SUCCEED) {
+	DEBUG1(DBG_DRIVER_LO, "no MXs for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
-	*defer_p = TRUE;
 	xfree(strip_target);
+	_res.options = save_res_options;
 
-        return FALSE;
+        return ret;
     }
-    if (decode_cnames(mxdat) != DB_SUCCEED) {
+    if ((ret = decode_cnames(mxdat)) != DB_SUCCEED) {
+	DEBUG1(DBG_DRIVER_LO, "problem decoding cnames for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
-	*defer_p = TRUE;
 	xfree(strip_target);
+	_res.options = save_res_options;
 
-        return FALSE;
+        return ret;
     }
-    if (make_mx_hints(mxdat) != DB_SUCCEED) {
+    if ((ret = make_mx_hints(mxdat)) != DB_SUCCEED) {
+	DEBUG1(DBG_DRIVER_LO, "problem finding MX hints for %s.\n", strip_target);
 	mx_work_area_destructor(mxdat, FALSE);
-	*defer_p = TRUE;
 	xfree(strip_target);
+	_res.options = save_res_options;
 
-        return FALSE;
+        return ret;
     }
-    ret = (mxdat->local_precedence >= 0); /* we found ourselves.... */
+    ret = (mxdat->local_precedence >= 0) ? (DB_SUCCEED + 1) : DB_SUCCEED;
 
     mx_work_area_destructor(mxdat, FALSE);
     xfree(strip_target);
@@ -601,7 +614,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
     static char *orig_target = NULL;    /* Original copy of target */
     static int no_server = FALSE;       /* TRUE if no server process */
     static int found_server = FALSE;    /* TRUE if server process found */
-    int success;                        /* value to return */
+    int result;                        /* value to return */
     int inverting = FALSE;              /* trying inverted address */
     struct mx_work_area * mxdat;        /* Work area for MX operations */
     char * gateway = NULL;
@@ -708,9 +721,9 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
         }
 
         mxdat = mx_work_area_constructor(priv, error_p, what, flags, &found_server, &no_server);
-        success = full_mx_lookup(target, mxdat, (gateway == NULL), priv);
+        result = full_mx_lookup(target, mxdat, (gateway == NULL), priv);
 
-        if (success == DB_SUCCEED) {
+        if (result == DB_SUCCEED) {
             /*
              * if the target is not in one of the required domains, then don't
              * match it. For systems behind firewalls who "see" systems on the
@@ -738,12 +751,12 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
             return DB_SUCCEED;
         }
     
-        if ((success == DB_NOMATCH) && (mxdat->UK_MX_rejected)) {
+        if ((result == DB_NOMATCH) && (mxdat->UK_MX_rejected)) {
             if ((priv->uk_greybook_transport  != NULL) ||
                 (priv->uk_greybook_hosts_list != NULL) ||
                 (priv->uk_greybook_hosts_file != NULL)) {
-                success = route_by_greybook(mxdat); /* Try to send by greybook */
-                if (success == DB_SUCCEED) {
+                result = route_by_greybook(mxdat); /* Try to send by greybook */
+                if (result == DB_SUCCEED) {
                     DEBUG(DBG_ROUTE_LO, "Matched by greybook lookups\n");
                     if ((priv->uk_greybook_transport == NULL) ||
                         ((rt_info->transport = find_transport(priv->uk_greybook_transport)) == NULL)) {
@@ -751,7 +764,7 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
                                                                priv->uk_greybook_transport ? 
                                                                priv->uk_greybook_transport :
                                                                "(not specified)");
-                        success = DB_FAIL;
+                        result = DB_FAIL;
                     } else {
                         if (priv->rewrite_headers >= (inverting ? BIND_REWRITE_IFREQUIRED : BIND_REWRITE_ALWAYS))
 			    do_header_rewrites(raw_target, 
@@ -773,13 +786,13 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
                     }
                 }
             }
-            if (success == DB_NOMATCH) {
+            if (result == DB_NOMATCH) {
                 if (priv->uk_greybook_relay != NULL) {
                     struct mx_work_area * mxdat2;
         
                     mxdat2 = mx_work_area_constructor(priv, error_p, what, flags, &found_server, &no_server);
-                       success = full_mx_lookup(priv->uk_greybook_relay, mxdat2, FALSE, priv);
-                    if (success == DB_SUCCEED) {
+                       result = full_mx_lookup(priv->uk_greybook_relay, mxdat2, FALSE, priv);
+                    if (result == DB_SUCCEED) {
                         if (gateway == NULL) {
                             if (priv->rewrite_headers >= (inverting ? BIND_REWRITE_IFREQUIRED : BIND_REWRITE_ALWAYS))
                                 do_header_rewrites(raw_target, mxdat->full_target);
@@ -800,20 +813,20 @@ bind_addr_work(raw_target, flags, priv, what, rt_info, error_p)
                     /* We have a positive match, but can't route to it, so there is no
                      * point looking further
                      */
-                    success = DB_NEGMATCH;
+                    result = DB_NEGMATCH;
                 }
             }
         }
-        if ((success != DB_NOMATCH) || (mxdat->UK_MX_rejected)) {
+        if ((result != DB_NOMATCH) || (mxdat->UK_MX_rejected)) {
             /* We return if an address was not matched for UK reasons because
              * we know that we have actually matched the address - we just are
              * unable to route to it.
              */
             mx_work_area_destructor(mxdat, FALSE);
-            if (success == DB_NEGMATCH)
+            if (result == DB_NEGMATCH)
                 return DB_NOMATCH;
             else
-                return success;
+                return result;
         }
         mx_work_area_destructor(mxdat, FALSE);
 
@@ -870,34 +883,34 @@ static int route_by_greybook(mxdat)
         }
     }
     if (mxdat->priv->uk_greybook_hosts_file != NULL) {
-        int success;
+        int result;
         char * db_ptr;
         char *error_text;
         char *return_value;
 
         DEBUG1(DBG_ROUTE_HI, "Opening greybook hosts file %s\n", 
                mxdat->priv->uk_greybook_hosts_file);
-        success = open_database(mxdat->priv->uk_greybook_hosts_file,
-				mxdat->priv->uk_greybook_hosts_proto,
-				(int) mxdat->priv->uk_greybook_host_retries,
-				(int) mxdat->priv->uk_greybook_host_interval,
-				(struct stat *)NULL, 
-				&db_ptr,
-				&error_text);
-        if (success != FILE_SUCCEED) {  /* If DB open fails... */
+        result = open_database(mxdat->priv->uk_greybook_hosts_file,
+			       mxdat->priv->uk_greybook_hosts_proto,
+			       (int) mxdat->priv->uk_greybook_host_retries,
+			       (int) mxdat->priv->uk_greybook_host_interval,
+			       (struct stat *)NULL, 
+			       &db_ptr,
+			       &error_text);
+        if (result != FILE_SUCCEED) {  /* If DB open fails... */
             *mxdat->error_p = open_failed(mxdat->what, 
                                           mxdat->priv->uk_greybook_hosts_file,
                                           error_text);
-            return success;
+            return result;
         }
         DEBUG1(DBG_ROUTE_MID, "Looking up greybook host %s\n", mxdat->full_target);
-        success = lookup_database(db_ptr,
-                                  mxdat->full_target,
-                                  &return_value, 
-                                  &error_text);
+        result = lookup_database(db_ptr,
+				 mxdat->full_target,
+				 &return_value, 
+				 &error_text);
         DEBUG(DBG_ROUTE_HI, "Closing greybook hosts file.\n");
         close_database(db_ptr);
-        if (success == DB_SUCCEED) 
+        if (result == DB_SUCCEED) 
             return DB_SUCCEED;          /* Pushing it but OK here! */
     }
     return DB_NOMATCH;
@@ -941,27 +954,27 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
      int allow_mangling;
      struct bindlib_private *priv; 
 {
-    int success;                        /* how have we done? */
+    int result;                        /* how have we done? */
 
-    success = find_mx_records(target, mxdat, allow_mangling);
-    if (success != DB_SUCCEED) {
-        return success;                 /* Actually this is failure! */
+    result = find_mx_records(target, mxdat, allow_mangling);
+    if (result != DB_SUCCEED) {
+        return result;
     }
 
-    success = decode_cnames(mxdat);
-    if (success != DB_SUCCEED) {
-        return success;
+    result = decode_cnames(mxdat);
+    if (result != DB_SUCCEED) {
+        return result;
     }
 
-    success = make_mx_hints(mxdat);
-    if (success != DB_SUCCEED) {
-        return success;
+    result = make_mx_hints(mxdat);
+    if (result != DB_SUCCEED) {
+        return result;
     }
 
     if (mxdat->mx_hints == NULL) {
-        success = handle_null_mxs(mxdat, priv);
-        if (success != DB_SUCCEED) {
-            return success;
+        result = handle_null_mxs(mxdat, priv);
+        if (result != DB_SUCCEED) {
+            return result;
         }
         if (mxdat->priv->fallback_gateway) 
             add_mx_hint(&(mxdat->mx_hints), BIND_FALLBACK_PREFERENCE, 
@@ -971,15 +984,15 @@ static int full_mx_lookup (target, mxdat, allow_mangling, priv)
             add_mx_hint(&(mxdat->mx_hints), BIND_FALLBACK_PREFERENCE, 
                         mxdat->priv->fallback_gateway, 1);
         /* Can only filter MX records if there are some! */
-        success = filter_mx_hints(mxdat);
-        if (success != DB_SUCCEED) {
-            return success;
+        result = filter_mx_hints(mxdat);
+        if (result != DB_SUCCEED) {
+            return result;
         }
     }
 
-    success = get_addr_hints(mxdat);
-    if (success != DB_SUCCEED) {
-        return success;
+    result = get_addr_hints(mxdat);
+    if (result != DB_SUCCEED) {
+        return result;
     }
 
     if (mxdat->mx_hints == NULL) {
@@ -1006,11 +1019,11 @@ static int find_mx_records (target, mxdat, allow_mangling)
      int allow_mangling;
 {
     char * cur_target = target;
-    int success;
+    int result;
 
-    success = do_straight_lookup(cur_target, mxdat);
+    result = do_straight_lookup(cur_target, mxdat);
 
-    if ((success == DB_NOMATCH) && allow_mangling) {
+    if ((result == DB_NOMATCH) && allow_mangling) {
         char * w;                       /* Working string */
         static struct str p;            /* region for building new string */
         static int initialised = FALSE; /* Have I initialised p yet? */
@@ -1027,8 +1040,8 @@ static int find_mx_records (target, mxdat, allow_mangling)
                 str_cat(&p, ".");       /* Add a dot */
                 str_cat(&p, w);         /* Append domain */
                 cur_target = STR(&p);
-                success = do_straight_lookup(cur_target, mxdat);
-                if (success != DB_NOMATCH)
+                result = do_straight_lookup(cur_target, mxdat);
+                if (result != DB_NOMATCH)
 			break;			/* Fall out end if not not found! */
             }
         }
@@ -1037,10 +1050,10 @@ static int find_mx_records (target, mxdat, allow_mangling)
         xfree(mxdat->matched_target);
         mxdat->matched_target = NULL;
     }
-    if (success == DB_SUCCEED) {
+    if (result == DB_SUCCEED) {
         mxdat->matched_target = COPY_STRING(cur_target);
     }
-    return success;
+    return result;
 }
 
 
@@ -1052,16 +1065,16 @@ static int do_straight_lookup(target, mxdat)
      char * target;
      struct mx_work_area * mxdat;
 {
-    int success;
+    int result;
     char * error_msg;
 
     DEBUG1(DBG_DRIVER_MID, "bindlib: MX lookup for %s\n", target);
-    success =  get_records(target, 
-                           T_MX, 
-                           mxdat->mx_rrs, 
-                           &(mxdat->mx_size), 
-                           &error_msg);
-    switch (success) {
+    result = get_records(target, 
+			 T_MX, 
+			 mxdat->mx_rrs, 
+			 &(mxdat->mx_size), 
+			 &error_msg);
+    switch (result) {
 
       case DB_SUCCEED:
         *mxdat->found_server = TRUE;
@@ -1076,9 +1089,9 @@ static int do_straight_lookup(target, mxdat)
 	     *    address we are in real trouble here - nothing will get done by
 	     *    the router 
 	     */
-	    success = FILE_NOMATCH;
+	    result = FILE_NOMATCH;
 	} else {
-	    success = DB_AGAIN;
+	    result = DB_AGAIN;
 	}
 	break;
 
@@ -1088,7 +1101,7 @@ static int do_straight_lookup(target, mxdat)
         break;
   
     }
-    return success;
+    return result;
 }
 
 
@@ -1097,18 +1110,17 @@ static int do_straight_lookup(target, mxdat)
  * decode_cnames        - Decode any cnames returned by earlier
  *                        lookups.
  *
- * If there are CNAME records in the set returned, then we look
- * up on the domain name pointed to by those records.  We iterate
- * this until we get a proper match (maybe we should have a
- * CNAME limit here - someone could make a loop in their DNS
- * entry??)
+ * If there are CNAME records in the set returned, then we look up on the
+ * domain name pointed to by those records.  We iterate this, with a maximum of
+ * cname_limit iterations (normally 1 -- the variable is a hack to assist
+ * broken DNS configurations), until we get a proper match.
  */
 static int decode_cnames(mxdat)
      struct mx_work_area * mxdat;
 {
     struct rr_iterator mx_it;
     RR *mx_rr;
-    int success;
+    int result;
     int num_iterations = 0;
 
     do {                                /* Iterate through any CNAMEs */
@@ -1139,10 +1151,10 @@ static int decode_cnames(mxdat)
              * actually an alias for some other domain name. The query
              * should be repeated with the canonical domain name.
              */
-            success =  find_mx_records(nambuf, mxdat, FALSE);
-            if (success != DB_SUCCEED) {
+            result =  find_mx_records(nambuf, mxdat, FALSE);
+            if (result != DB_SUCCEED) {
                 /* Cleanup code - deallocate storage */
-                return success;         /* Actually this is failure! */
+                return result;
             }
         }
     } while (mx_rr != NULL);
@@ -1163,7 +1175,7 @@ static int handle_null_mxs(mxdat, priv)
      struct mx_work_area * mxdat;
      struct bindlib_private *priv;  
 {
-    int success;
+    int result;
 
     if (mxdat->flags & BIND_MX_ONLY) {
         /* We need MX records.  We have none - hence failed! */
@@ -1189,8 +1201,8 @@ static int handle_null_mxs(mxdat, priv)
         add_mx_hint(&(mxdat->mx_hints), 0,
                     mxdat->matched_target,
                     TRUE);
-        success = get_addr_hints(mxdat);
-        if (success == DB_SUCCEED) {
+        result = get_addr_hints(mxdat);
+        if (result == DB_SUCCEED) {
             /* OK we have an implicit MX with address records (we hope!)
              * but we may not have the FQDN as yet.  So lets get the
              * FQDN out of the address hint.  Just remember here that
@@ -1213,7 +1225,7 @@ static int handle_null_mxs(mxdat, priv)
             }
         }
 
-        return success;                 /* A touch optomistic! */
+        return result;                 /* A touch optomistic! */
 #undef mx_hint
     }
 }
@@ -1232,7 +1244,7 @@ static int make_mx_hints(mxdat)
 {
     struct rr_iterator mx_it;
     RR * mx_rr;
-    int success;
+    int result;
 
     rewindrr(&mx_it, mxdat->mx_rrs, mxdat->mx_size);
     while ((mx_rr = getnextrr(&mx_it, FALSE))) {
@@ -1240,8 +1252,8 @@ static int make_mx_hints(mxdat)
             char * name;
             int precedence;
 
-            success = decode_mx_rr(mx_rr, &mx_it, &name, &precedence);
-            if (success != DB_SUCCEED) {
+            result = decode_mx_rr(mx_rr, &mx_it, &name, &precedence);
+            if (result != DB_SUCCEED) {
                 *(mxdat->error_p) = packet_error(mxdat->what, "MX", mxdat->matched_target);
                 return DB_AGAIN;
             }
@@ -1355,7 +1367,7 @@ static int get_addr_hints(mxdat)
     struct rr_iterator a_it;
     RR * a_rr;
     struct transport_hints ** mx_a;
-    int success;
+    int result;
     char * error_msg;
     int dns_timeout = 0;
 
@@ -1381,8 +1393,8 @@ static int get_addr_hints(mxdat)
 #define mx_hint ((struct mx_transport_hint *)((*mx_a)->private))
         if (! mx_hint->ipaddrs) {
             /* Go out and get an A record */
-            success = find_a_records(mx_hint, mx_hint->exchanger, &error_msg);
-            switch (success) {
+            result = find_a_records(mx_hint, mx_hint->exchanger, &error_msg);
+            switch (result) {
         
               case DB_SUCCEED:
                 mx_a = &(*mx_a)->succ;
@@ -1395,7 +1407,7 @@ static int get_addr_hints(mxdat)
                 break;
 
               case FILE_NOMATCH:
-                success = FILE_AGAIN;
+                result = FILE_AGAIN;
                 /* FALLTHRU */
 
               case DB_AGAIN:
@@ -1893,21 +1905,21 @@ find_a_records(mx_hint, target, error)
     char *target;
     char **error;
 {
-    int success;
     HEADER *a_rrs;
     int a_size;
     struct rr_iterator a_it;
     RR *a_rr;
-    int result = DB_NOMATCH;
+    int result;
 
     a_rrs = (HEADER *)xmalloc(MAXPACKET);
-    success = get_records(target, T_A, a_rrs, &a_size, error);
+    result = get_records(target, T_A, a_rrs, &a_size, error);
 
-    if (success != DB_SUCCEED) {
+    if (result != DB_SUCCEED) {
         xfree((char *)a_rrs);
-        return success;
+        return result;
     }
 
+    result = DB_NOMATCH;
     rewindrr(&a_it, a_rrs, a_size);
     while ((a_rr = getnextrr(&a_it, FALSE))!=0) {
         if (a_rr->rr_type == T_A) {
