@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:config.h,v 1.43 1997/03/07 03:26:35 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:config.h,v 1.44 1997/04/21 17:51:15 woods Exp"
  */
 
 /*
@@ -1081,6 +1081,10 @@ ${if def:input_addr\
  * for information on what the command and filename strings look
  * like.
  *
+ * Note that LOCAL_MAIL_FILE may be automatically defined in either the
+ * system configuration or the local configuration by defining the
+ * variable MAILBOX_DIR, in which case "/${lc:user}" is appended.
+ *
  * Note that the program called as LOCAL_MAIL_COMMAND must enforce all
  * of the local requirements of a ``unix'' mailbox format, which are
  * normally as follows (from the 4.4bsd mail.local(8) manual page):
@@ -1142,6 +1146,18 @@ ${if def:input_addr\
 # endif
 #endif
 
+/*
+ * if smail is doing delivery to mailbox files directly, these files will
+ * be opened with this group.
+ */
+#ifndef LOCAL_MAIL_GROUP
+# if defined(UNIX_SYS5) && !defined(UNIX_SUN_OS_4) && !defined(UNIX_BSD)
+#  define LOCAL_MAIL_GROUP	"mail"	/* under System V use group "mail" */
+# else
+#  define LOCAL_MAIL_GROUP	NULL	/* otherwise, no group in particular */
+# endif
+#endif
+
 /*
  * Setup for the queued uux transport
  *
