@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:default.c,v 1.31 1997/04/12 05:41:28 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:default.c,v 1.32 1997/04/21 17:54:14 woods Exp"
  */
 
 /*
@@ -679,7 +679,7 @@ static struct router bind_hosts_router = {
     "bind",				/* use the bind driver */
     SUCC,				/* point to next router */
     BIND_DEFER_NO_CONN |		/* defer if connection not successful */
-#ifdef USE_STRICT_MX
+#ifdef USE_STRICT_MX			/* contravenes RFC 974's advice */
 	BIND_MX_ONLY |			/* only match MX'ed hosts */
 #endif
 	BIND_DOMAIN_REQUIRED,		/* don't match un-qualified names */
@@ -882,20 +882,14 @@ static struct transport local_transport = {
     NULL,				/* error transport */
     (char *)&local_transport_priv,	/* local private configuration */
 };
-#undef SUCC
-#define SUCC (&local_transport)		/* setup for forward link */
 
-#else	/* LOCAL_MAIL_COMMAND */
+#else	/* !LOCAL_MAIL_COMMAND */
 
 static struct appendfile_private local_transport_priv = {
     LOCAL_MAIL_FILE,			/* e.g.: /usr/spool/mail/${lc:user} */
     NULL,				/* directory does not apply */
     NULL,				/* run as the particular user */
-# if defined(UNIX_SYS5) && !defined(UNIX_SUN_OS) && !defined(UNIX_BSD)
-    "mail",				/* under System V use group "mail" */
-# else
-    NULL,				/* otherwise, no group in particular */
-# endif
+    LOCAL_MAIL_GROUP,			/* group used when opening mailboxes */
     NULL,				/* no message prefix */
     "\n",				/* end message with extra newline */
     LOCAL_MAIL_MODE,			/* file creation mode */
@@ -915,9 +909,11 @@ static struct transport local_transport = {
     NULL,				/* error transport */
     (char *)&local_transport_priv,	/* local private configuration */
 };
+
+#endif	/* LOCAL_MAIL_COMMAND */
+
 #undef SUCC
 #define SUCC (&local_transport)		/* setup for forward link */
-#endif	/* LOCAL_MAIL_COMMAND */
 
 
 /*
@@ -1469,6 +1465,9 @@ static struct transport uucp_zone_bind_smtp_transport = {
     "tcpsmtp",				/* uses smtp transport driver */
     SUCC,				/* point to next transport */
     BIND_DEFER_NO_CONN |		/* this makes more sense for modern DNS hosts */
+#ifdef USE_STRICT_MX			/* contravenes RFC 974's advice */
+	BIND_MX_ONLY |			/* only match MX'ed hosts */
+#endif
 	BIND_DOMAIN_REQUIRED |
 	TCPSMTP_USE_BIND |
 	UUCP_XFORM |			/* this probably isn't of much use.... */
@@ -1488,6 +1487,9 @@ static struct transport local_bind_smtp_transport = {
     "tcpsmtp",				/* uses smtp transport driver */
     SUCC,				/* point to next transport */
     BIND_DEFER_NO_CONN |		/* this makes more sense for modern DNS hosts */
+#ifdef USE_STRICT_MX			/* contravenes RFC 974's advice */
+	BIND_MX_ONLY |			/* only match MX'ed hosts */
+#endif
 	BIND_DOMAIN_REQUIRED |
 	TCPSMTP_USE_BIND |
 	LOCAL_XFORM |			/* this probably isn't of much use.... */
