@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:modes.c,v 1.61 1997/04/12 01:57:11 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:modes.c,v 1.62 1997/04/16 02:04:10 woods Exp"
  */
 
 /*
@@ -120,6 +120,13 @@
 # define GID_NOBODY		GID_MAX
 #endif
 
+/* Check your kernel for the configured value, and also verify it has been set
+ * appropriately in <sys/socket.h>.  There's no way to check automatically. (?)
+ */
+#ifndef SOMAXCONN
+# define SOMAXCONN	5
+#endif
+
 /* variables exported from this file */
 int daemon_pid = 0;
 
@@ -1039,8 +1046,12 @@ daemon_mode()
     (void) signal(SIGALRM, daemon_sigalrm);
     (void) alarm(queue_interval);
 
+    /* fire up the smtp socket */
+    if (listen(ls, SOMAXCONN) == -1) {
+	panic(EX_OSERR, "listen(): %s", strerror(errno));
+	/* NOTREACHED */
+    }
     /* loop processing connect requests or alarm signals */
-    (void) listen(ls, 5);
     for (;;) {
 	int len = sizeof(from);
 
