@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:smailconf.c,v 1.32 1997/04/12 20:56:19 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:smailconf.c,v 1.33 1997/04/16 02:08:27 woods Exp"
  */
 
 /*
@@ -708,7 +708,7 @@ print_config_all()
 	case t_long:
 	case t_interval:
 	case t_double:
-	case t_infoproc:
+	case t_infoproc:		/* XXX should have t_proc too? */
 	    print_config_variable(t->name);
 	    break;
 	default:
@@ -1012,10 +1012,10 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 	    break;
 
 	case t_proc:
-	    if ((error = (*((char *(*)())t->uptr))(sp, attrs)))
-	    {
+	    if ((error = (*((char *(*)())t->uptr))(sp, attrs))) {
 		return error;
 	    }
+	    break;
 
 	case t_infoproc:
 	    return xprintf("%s: read-only attribute", attrs->name);
