@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:smtprecv.c,v 1.31 1997/04/12 21:07:35 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:smtprecv.c,v 1.33 1997/04/21 18:06:20 woods Exp"
  */
 
 /*
@@ -32,6 +32,7 @@
 # include "extern.h"
 # include "debug.h"
 # include "exitcodes.h"
+# include "error.h"
 #endif
 
 #ifdef HAVE_RFC1413
@@ -108,9 +109,7 @@ static char *help_msg[] = {
     "250-   EHLO hostname                   - startup with extension info",
     "250-   MAIL FROM:<sender-address>      - start transaction from sender",
     "250-   RCPT TO:<recipient-address>     - name recipient for message",
-#ifndef NO_SMTP_VRFY
     "250-   VRFY <address>                  - verify deliverability of address",
-#endif
 #ifndef NO_SMTP_EXPN
     "250-   EXPN <address>                  - expand mailing list address",
 #endif
@@ -607,15 +606,16 @@ receive_smtp(in, out)
 	    break;
 
 	case VRFY_CMD:
+#define SMTP_RPLY_252	"252 Cannot VRFY user, but will take message for this user and attempt delivery.\r\n"
 	    if (out) {
 #ifdef NO_SMTP_VRFY			/* XXX should be a variable ala smtp_debug */
-		fprintf(out, "502 Command not implemented\r\n");
+		fprintf(out, SMTP_RPLY_252);
 #else
 		if (smtp_info) {
 		    strip_rfc822_comments(data);
 		    verify_addr(data, out, 0);
 		} else {
-		    fprintf(out, "502 Command disabled\r\n");
+		    fprintf(out, SMTP_RPLY_252);
 		}
 #endif
 		fflush(out);
@@ -784,8 +784,8 @@ read_smtp_command(f, out)
 	"MAIL FROM:",	sizeof("MAIL FROM:")-1,	MAIL_CMD,
 	"RCPT TO:",	sizeof("RCPT TO:")-1,	RCPT_CMD,
 	"DATA",		sizeof("DATA")-1,	DATA_CMD,
-	"VRFY",		sizeof("VRFY")-1,	VRFY_CMD, /* need it here */
-	"EXPN",		sizeof("EXPN")-1,	EXPN_CMD, /* need it here */
+	"VRFY",		sizeof("VRFY")-1,	VRFY_CMD,
+	"EXPN",		sizeof("EXPN")-1,	EXPN_CMD,
 	"QUIT",		sizeof("QUIT")-1,	QUIT_CMD,
 	"RSET",		sizeof("RSET")-1,	RSET_CMD,
 	"NOOP",		sizeof("NOOP")-1,	NOOP_CMD,
@@ -905,15 +905,12 @@ verify_addr(in_addr, out, rcpt_p)
     struct addr *defer = NULL;		/* temporarily unverifiable addr */
     struct addr *fail = NULL;		/* unverified addr */
     char *error;			/* hold error message */
-    int error_code;			/* reply code for negative result */
 
-    error_code = rcpt_p ? 501 : 550;
     addr->in_addr = in_addr;		/* setup the input addr structure */
     /* build the mungeable addr string */
     addr->work_addr = preparse_address(in_addr, &error);
     if (addr->work_addr == NULL) {
-	fprintf(out, "%d '%s' malformed address; %s.\r\n",
-		error_code, in_addr, error);
+	fprintf(out, "501 '%s' malformed address; %s.\r\n", in_addr, error);
 	return 0;
     }
 
@@ -929,27 +926,51 @@ verify_addr(in_addr, out, rcpt_p)
     verify_addr_list(addr, &okay, &defer, &fail);
 
     if (okay) {
-	fprintf(out, "250 <%s> Recipient Okay.\r\n", addr->work_addr);
+	fprintf(out, "250 <%s@%s> Recipient Okay.\r\n", addr->work_addr, addr->target);
  	return 1;
     } else if (defer) {
- 	fprintf(out, "%d '%s' cannot verify at this time; %s.\r\n", 
- 		rcpt_p ? 250 : 550, in_addr, defer->error->message);
+ 	fprintf(out, "450 <%s@%s> cannot verify at this time: (ERR_%03ld) %s.\r\n",
+ 		addr->work_addr, addr->target,
+		defer->error->info & ERR_MASK, defer->error->message);
  	return 1;
     } else if (fail) {
- 	fprintf(out, "%d '%s' not matched; %s.\r\n",
- 		error_code, in_addr, fail->error->message);
+	int error_code;			/* reply code for negative result */
+
+	switch (fail->error->info & ERR_MASK) {
+	case ERR_104:
+	    error_code = 551;		/* mailbox not local */
+	    break;
+	case ERR_111:
+	    error_code = 501;		/* syntax error in parameter */
+	    break;
+	case ERR_107:
+	case ERR_109:
+	    error_code = 451;		/* local error in processing */
+	    break;
+	case ERR_100:
+	    error_code = 550;		/* mailbox not found */
+	    break;
+	default:
+	    error_code = 553;		/* use this to flag un-xlated errors */
+	    break;
+	}
+ 	fprintf(out, "%d <%s@%s> not matched: %s.\r\n",
+ 		error_code, addr->work_addr, addr->target, fail->error->message);
 	if (rcpt_p) {
-	    write_log(LOG_SYS, "'%s' recipient not matched: %s",
-		      in_addr, fail->error->message);
+	    write_log(LOG_SYS, "remote %s: '%s' <%s@%s> recipient not matched: %s",
+		      sender_host_addr ? sender_host_addr : "<no-sender-addr>",
+		      in_addr, addr->work_addr, addr->target, fail->error->message);
 	}
  	return 0;
     } else {
   	/* hmmm, it should have been in one of the lists */
- 	fprintf(out, "%d '%s' does not match anything.\r\n",
- 		error_code, in_addr);
+ 	fprintf(out, "550 <%s@%s> does not match anything!\r\n",
+		addr->work_addr, addr->target); /* mailbox not found */
 	if (rcpt_p) {
 	    /* XXX do we really need to log these? */
-	    write_log(LOG_SYS, "'%s' recipient not matched.", in_addr);
+	    write_log(LOG_SYS, "remote %s: '%s' <%s@%s> recipient not matched.",
+		      sender_host_addr ? sender_host_addr : "(no-sender-addr)",
+		      in_addr, addr->work_addr, addr->target);
 	}
  	return 0;
     }
