@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_93:smtplib.c,v 1.22 1997/04/12 01:47:58 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_94:smtplib.c,v 1.24 1997/04/19 22:22:38 woods Exp"
  */
 
 /*
@@ -287,15 +287,19 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		if (*cp == '\n')
 		    ++cp;
 	    }
-#ifndef NO_LOG_EHLO
-	    write_log(LOG_SYS, "destination supports esmtp%s%s%s%s%s%s",
+# ifndef NODEBUG
+	    /* not worth defining DEBUG6()! */
+	    if (debug > DBG_REMOTE_MID && errfile) {
+		dprintf(errfile, "destination supports ESMTP%s%s%s%s%s%s",
 		      smtpb->smtp_flags & ESMTP_8bitmime ? " 8BITMIME" : "",
 		      smtpb->smtp_flags & ESMTP_size ? " SIZE" : "",
 		      smtpb->smtp_flags & ESMTP_pipelining ? " PIPELINING" : "",
 		      smtpb->smtp_flags & ESMTP_verbose ? " XVRB" : "",
 		      smtpb->smtp_flags & ESMTP_one ? " XONE" : "",
 		      smtpb->smtp_flags & ESMTP_queue ? " XQUE" : "");
-#endif /* not NO_LOG_EHLO */
+		fflush(errfile);
+	    }
+# endif
 	    if (smtpb->smtp_flags & ESMTP_verbose
 		&& debug >= DBG_DRIVER_MID) {
 		smtp_out.i = 0;
@@ -312,10 +316,10 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	  malformed_ehlo_reply:
 	    /* This seems to be a reasonable way
 	       to handle malformed EHLO replies: */
-#ifndef NO_LOG_EHLO
+# ifndef NO_LOG_EHLO
 	    write_log(LOG_SYS, "destination supports esmtp, but is buggy (%s)",
 		      reply_text);
-#endif /* not NO_LOG_EHLO */
+# endif /* not NO_LOG_EHLO */
 	    return SMTP_SUCCEED;
 	}
     }
@@ -343,10 +347,8 @@ smtp_startup(smtpb, error_p, try_ehlo)
     }
     if (reply == REPLY_SEQUENCE_ERROR) {
 	/* ignore 503 Bad sequence of commands after EHLO/RSET/HELO */
-#ifndef NO_LOG_EHLO
 	write_log(LOG_SYS, "503 after EHLO/RSET/HELO (%s)",
 		  reply_text);
-#endif /* not NO_LOG_EHLO */
     } else if (reply != REPLY_OK) {
 #ifdef HAVE_EHLO
 	if (! tried_rset) {
@@ -361,27 +363,23 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		/* remote SMTP closed, try again later */
 		*error_p = try_again(smtpb->tp, reply_text);
 		return SMTP_AGAIN;
-	    } else if (reply == REPLY_OK) {
-	    } else {
+	    } else if (reply != REPLY_OK) {
 		/* Some gateways don't accept any commands before they get
-		   a HELO, not even RSET.  Fortunately it is usually safe
-		   to ignore the error messages. */
-#ifndef NO_LOG_EHLO
+		 * a HELO, not even RSET.  Fortunately it is usually safe
+		 * to ignore the error messages. */
 		write_log(LOG_SYS, "unexpected response to RSET (%s)",
 			  reply_text);
-#endif /* not NO_LOG_EHLO */
 	    }
 	    /* the RSET command has been accepted; try again with HELO */
 	    tried_rset = 1;
 	    goto try_helo;	/* GASP!!! */
-	} else {
+	} else
 #endif /* HAVE_EHLO */
-	/* fatal error, return message to sender */
-	*error_p = fatal_error(smtpb->tp, reply_text);
-	return SMTP_FAIL;
-#ifdef HAVE_EHLO
+	{
+	    /* fatal error, return message to sender */
+	    *error_p = fatal_error(smtpb->tp, reply_text);
+	    return SMTP_FAIL;
 	}
-#endif
     }
 
     /* connection established */
