@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_93:verify.c,v 1.7 1997/04/12 21:02:30 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_94:verify.c,v 1.8 1997/04/21 18:04:28 woods Exp"
  */
 
 /*
@@ -25,6 +25,8 @@
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
+#include "exitcodes.h"
+#include "lookup.h"
 #ifndef DEPEND
 # include "extern.h"
 # include "debug.h"
@@ -136,25 +138,66 @@ verify_addr_list(in, okay, defer, fail)
                 cur->local_name = copy(cur->target);
 		cur->work_addr = cur->remainder;
 		next = cur;
-	    } else {
-		int binddefer = 0;
+	    } else if (!in->parent && sender_host_addr && smtp_remote_allow &&
+		       !match_ip(sender_host_addr, smtp_remote_allow)) {
+		int mxresult;
 
-	        if (!in->parent && sender_host_addr && smtp_remote_allow &&
-		    !match_ip(sender_host_addr, smtp_remote_allow) &&
-		    !bind_check_if_local_mxs(cur, &binddefer)
-		    ) {
-		    if (binddefer) {
-			cur->succ = *defer;
-			*defer = cur;
-		    } else {
+		if ((mxresult = bind_check_if_local_mxs(cur)) >= (DB_SUCCEED + 1)) {
+		    /* it's OK -- we MX for them... */
+		    cur->succ = remote;
+		    remote = cur;
+		} else {
+		    switch (mxresult) {
+		    case DB_SUCCEED:
 			illegal_relay_error(cur, sender_host_addr);
 			cur->succ = *fail;
 			*fail = cur;
+			break;
+		    case DB_AGAIN:	/* DNS lookup must be deferred */
+		    case FILE_AGAIN:
+			if (!cur->error) {
+			    char *error_text;
+
+			    error_text = xprintf("defer checking for MXs for %s",
+						 cur->in_addr);
+			    cur->error = note_error(ERR_DONTLOG | ERR_163, error_text);
+			}
+			cur->succ = *defer;
+			*defer = cur;
+			break;
+		    case DB_NOMATCH:	/* no such domain */
+		    case DB_FAIL:	/* bad DNS request? */
+			if (!cur->error) {
+			    char *error_text;
+
+			    error_text = xprintf("%s is not a vaild domain",
+						 cur->in_addr);
+			    cur->error = note_error(ERR_NPOWNER | ERR_164, error_text);
+			}	
+			cur->succ = *fail;
+			*fail = cur;
+			break;
+		    case FILE_FAIL:		/* major DNS error! */
+			if (!cur->error) {
+			    char *error_text;
+
+			    error_text = xprintf("error checking for MXs for %s",
+						 cur->in_addr);
+			    cur->error = note_error(ERR_NPOWNER | ERR_165, error_text);
+			}
+			cur->succ = *fail;
+			*fail = cur;
+			break;
+		    default:
+			panic(EX_SOFTWARE,
+			      "bind_check_if_local_mxs() impossible result %d.",
+			      mxresult);
+			/* NOTREACHED */
 		    }
-		} else {
-		    cur->succ = remote;
-		    remote = cur;
 		}
+	    } else {
+		cur->succ = remote;
+		remote = cur;
 	    }
 	    break;
 	}
@@ -227,9 +270,8 @@ match_ip(ip, pattern)
 
 
 static void
-illegal_relay_error(cur, sender_host_addr)
+illegal_relay_error(cur)
     struct addr *cur;
-    char *sender_host_addr;
 {
     char *error_text;
 
@@ -249,7 +291,6 @@ illegal_relay_error(cur, sender_host_addr)
      *      all valid local networks are listed properly in
      *      the smtp_remote_allow variable.
      */
-    error_text = xprintf("security violation: remote address not allowed from %s",
-			 sender_host_addr);
-    cur->error = note_error(ERR_DONTLOG | ERR_104, error_text);
+    error_text = xprintf("security violation: remote address not permitted");
+    cur->error = note_error(ERR_NPOSTMAST | ERR_104, error_text);
 }
